@@ -14,13 +14,14 @@ import javax.servlet.http.HttpSession;
 
 import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogue;
 import org.gcube.datacatalogue.ckanutillibrary.server.DataCatalogueFactory;
-import org.gcube.datacatalogue.ckanutillibrary.server.models.ResourceBean;
 import org.gcube.datacatalogue.ckanutillibrary.server.utils.SessionCatalogueAttributes;
 import org.gcube.datacatalogue.ckanutillibrary.server.utils.UtilMethods;
+import org.gcube.datacatalogue.ckanutillibrary.shared.ResourceBean;
 import org.gcube.datacatalogue.ckanutillibrary.shared.RolesCkanGroupOrOrg;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.CKanPublisherService;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.threads.AssociationToGroupAndNotifyThread;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.threads.WritePostCatalogueManagerThread;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.DiscoverTagsList;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.Utils;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.WorkspaceUtils;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.DatasetMetadataBean;
@@ -56,7 +57,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	// Logger
 	//private static final org.slf4j.Logger logger = LoggerFactory.getLogger(CKANPublisherServicesImpl.class);
 	private static final Log logger = LogFactoryUtil.getLog(CKANPublisherServicesImpl.class);
-	private static final String PRODUCT_URL_FIELD = "Product URL";
+	private static final String ITEM_URL_FIELD = "Item URL";
 
 	// map <orgName, scope>
 	private ConcurrentHashMap<String, String> mapOrganizationScope = new ConcurrentHashMap<String, String>();
@@ -202,6 +203,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				bean.setMaintainer(userOwner.getFullname());
 				bean.setMaintainerEmail(userOwner.getEmail());
 				bean.setOrganizationList(getUserOrganizationsListAdmin(userName, scope));
+				bean.setTagsVocabulary(discoverTagsVocabulary(scope));
 
 				// if the request comes from the workspace
 				if(folderId != null && !folderId.isEmpty()){
@@ -213,6 +215,9 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			}
 		}else{
 
+			logger.info("DEV MODE DETECTED");
+			Utils.getCurrentToken(getThreadLocalRequest(), true);
+
 			try{
 				bean = new DatasetMetadataBean();
 				bean.setId(folderId);
@@ -225,7 +230,8 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				bean.setAuthorEmail("costantino.perciante@isti.cnr.it");
 				bean.setMaintainer("Costantino Perciante");
 				bean.setMaintainerEmail("costantino.perciante@isti.cnr.it");
-				bean.setOrganizationList(getUserOrganizationsListAdmin(userName, Utils.getCurrentContext(getThreadLocalRequest(), true)));
+				//bean.setOrganizationList(getUserOrganizationsListAdmin(userName, Utils.getCurrentContext(getThreadLocalRequest(), true)));
+				bean.setOrganizationList(Arrays.asList(new OrganizationBean("preVRE", "prevre")));
 				bean.setOwnerIdentifier(userName);
 
 				if(folderId != null && !folderId.isEmpty()){
@@ -236,109 +242,131 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			}
 		}
 
+		logger.debug("Returning bean " + bean);
 		return bean;
 	}
 
+	/**
+	 * Discover from the IS the vocabulary of tags for this scope, if present
+	 * @return a list of tags vocabulary
+	 */
+	private List<String> discoverTagsVocabulary(String context) {
+
+		logger.debug("Looking for vocabulary of tags in this context " + context);
+		String keyPerVocabulary = UtilMethods.concatenateSessionKeyScope("TAGS_VOCABULARY", context); // TODO REMOVE
+
+		List<String> vocabulary = (List<String>) getThreadLocalRequest().getSession().getAttribute(keyPerVocabulary);
+		if(vocabulary != null){
+			vocabulary = new DiscoverTagsList(context).getTagsVocabulary();
+			getThreadLocalRequest().getSession().setAttribute(keyPerVocabulary, vocabulary);
+		}
+		
+		logger.debug("Vocabulary for tags is " + vocabulary);
+
+		return vocabulary;
+	}
+
 	@Override
-	public DatasetMetadataBean createCKanDataset(DatasetMetadataBean toCreate) {
+	public DatasetMetadataBean createCKanDataset(DatasetMetadataBean toCreate) throws Exception{
 
 		logger.debug("Request for creating a dataset with these information " + toCreate);
 
 		String userName = Utils.getCurrentUser(getThreadLocalRequest()).getUsername();
 
-		try{
+		String title = toCreate.getTitle();
+		String organizationNameOrId = toCreate.getSelectedOrganization();
+		String author = toCreate.getAuthorFullName();
+		String authorMail = toCreate.getAuthorEmail();
+		String maintainer = toCreate.getMaintainer();
+		String maintainerMail = toCreate.getMaintainerEmail();
+		long version = toCreate.getVersion();
+		String description = toCreate.getDescription();
+		String chosenLicense = toCreate.getLicense();
+		String licenseId = findLicenseIdByLicense(chosenLicense);
+		List<String> listOfTags = toCreate.getTags();
+		Map<String, List<String>> customFields = toCreate.getCustomFields();
 
-			String title = toCreate.getTitle();
-			String organizationNameOrId = toCreate.getSelectedOrganization();
-			String author = toCreate.getAuthorFullName();
-			String authorMail = toCreate.getAuthorEmail();
-			String maintainer = toCreate.getMaintainer();
-			String maintainerMail = toCreate.getMaintainerEmail();
-			long version = toCreate.getVersion();
-			String description = toCreate.getDescription();
-			String chosenLicense = toCreate.getLicense();
-			String licenseId = findLicenseIdByLicense(chosenLicense);
-			List<String> listOfTags = toCreate.getTags();
-			Map<String, List<String>> customFields = toCreate.getCustomFields();
-			boolean setPublic = toCreate.getVisibility();
+		// add Type for custom fields
+		if(toCreate.getChosenType() != null)
+			customFields.put("system:type", Arrays.asList(toCreate.getChosenType()));
 
-			// get the list of resources and convert to ResourceBean
-			List<ResourceBean> resources = null;
-			List<ResourceElementBean> resourcesToAdd = toCreate.getResources();
+		boolean setPublic = toCreate.getVisibility();
 
-			// we need to copy such resource in the .catalogue area of the user's ws
-			if(resourcesToAdd != null && !resourcesToAdd.isEmpty()){
-				resources = WorkspaceUtils.copyResourcesToUserCatalogueArea(toCreate.getId(), userName, toCreate);
-			}
+		// get the list of resources and convert to ResourceBean
+		List<ResourceBean> resources = null;
+		ResourceElementBean resourcesToAdd = toCreate.getResourceRoot();
 
-			logger.debug("The user wants to publish in organization with name " + organizationNameOrId);
-			String scope = getScopeFromOrgName(organizationNameOrId);
-			DataCatalogue utils = getCatalogue(scope);
-			String userApiKey = utils.getApiKeyFromUsername(userName);
-
-			String datasetId = utils.createCKanDatasetMultipleCustomFields
-					(userApiKey, title, null, organizationNameOrId, author, authorMail, maintainer, 
-							maintainerMail, version, description, licenseId, listOfTags, customFields, resources, setPublic);
-
-			if(datasetId != null){
-
-				logger.info("Dataset created!");
-				toCreate.setId(datasetId);
-
-				// retrieve the url
-				String datasetUrl =  utils.getUnencryptedUrlFromDatasetIdOrName(datasetId);
-				toCreate.setSource(datasetUrl);
-
-				// add also this information as custom field
-				Map<String, List<String>> addField = new HashMap<String, List<String>>();
-				addField.put(PRODUCT_URL_FIELD, Arrays.asList(datasetUrl));
-				utils.patchProductCustomFields(datasetId, userApiKey, addField);
-
-				// start a thread that will associate this dataset with the group
-				if(toCreate.getChosenProfile() != null || toCreate.getGroups() != null){
-
-					AssociationToGroupAndNotifyThread threadAssociationToGroup = 
-							new AssociationToGroupAndNotifyThread(
-									toCreate.getGroups(),
-									toCreate.getChosenProfile(), 
-									datasetUrl,
-									datasetId,
-									toCreate.getTitle(),
-									Utils.getCurrentUser(getThreadLocalRequest()).getFullname(),
-									userName, 
-									utils, 
-									organizationNameOrId,
-									getThreadLocalRequest()
-									);
-					threadAssociationToGroup.start();
-
-				}
-
-				// launch notification thread
-				WritePostCatalogueManagerThread threadWritePost = 
-						new WritePostCatalogueManagerThread(
-								userName,
-								scope, 
-								toCreate.getTitle(), 
-								datasetUrl, 
-								false, // send notification to other people 
-								toCreate.getTags(), 
-								Utils.getCurrentUser(getThreadLocalRequest()).getFullname()
-								);
-				threadWritePost.start();
-
-				return toCreate;
-			}else{
-				logger.error("Failed to create the dataset");
-			}
-		}catch(Exception e){
-			logger.error("Unable to create the dataset", e);
+		// we need to copy such resource in the .catalogue area of the user's ws
+		if(resourcesToAdd != null){
+			resources = WorkspaceUtils.copyResourcesToUserCatalogueArea(toCreate.getId(), userName, toCreate);
 		}
+
+		logger.debug("The user wants to publish in organization with name " + organizationNameOrId);
+		String scope = getScopeFromOrgName(organizationNameOrId);
+		DataCatalogue utils = getCatalogue(scope);
+		String userApiKey = utils.getApiKeyFromUsername(userName);
+
+		String datasetId = utils.createCKanDatasetMultipleCustomFields
+				(userApiKey, title, null, organizationNameOrId, author, authorMail, maintainer, 
+						maintainerMail, version, description, licenseId, listOfTags, customFields, resources, setPublic);
+
+		if(datasetId != null){
+
+			logger.info("Dataset created!");
+			toCreate.setId(datasetId);
+
+			// retrieve the url
+			String datasetUrl =  utils.getUnencryptedUrlFromDatasetIdOrName(datasetId);
+			toCreate.setSource(datasetUrl);
+
+			// add also this information as custom field
+			Map<String, List<String>> addField = new HashMap<String, List<String>>();
+			addField.put(ITEM_URL_FIELD, Arrays.asList(datasetUrl));
+			utils.patchProductCustomFields(datasetId, userApiKey, addField);
+
+			// start a thread that will associate this dataset with the group
+			if(/*toCreate.getChosenType() != null ||*/ toCreate.getGroups() != null){
+
+				AssociationToGroupAndNotifyThread threadAssociationToGroup = 
+						new AssociationToGroupAndNotifyThread(
+								toCreate.getGroups(),
+								null, //toCreate.getChosenType(), TODO
+								datasetUrl,
+								datasetId,
+								toCreate.getTitle(),
+								Utils.getCurrentUser(getThreadLocalRequest()).getFullname(),
+								userName, 
+								utils, 
+								organizationNameOrId,
+								getThreadLocalRequest()
+								);
+				threadAssociationToGroup.start();
+
+			}
+
+			// launch notification thread
+			WritePostCatalogueManagerThread threadWritePost = 
+					new WritePostCatalogueManagerThread(
+							userName,
+							scope, 
+							toCreate.getTitle(), 
+							datasetUrl, 
+							false, // send notification to other people 
+							toCreate.getTags(), 
+							Utils.getCurrentUser(getThreadLocalRequest()).getFullname()
+							);
+			threadWritePost.start();
+
+			return toCreate;
+		}else{
+			logger.error("Failed to create the dataset");
+		}
+
 		return null;
 	}
 
 	@Override
-	public ResourceElementBean addResourceToDataset(ResourceElementBean resource, String datasetId) {
+	public ResourceElementBean addResourceToDataset(ResourceElementBean resource, String datasetId) throws Exception{
 		String username = Utils.getCurrentUser(getThreadLocalRequest()).getUsername();
 
 		logger.debug("Incoming request for creating new resource for dataset with id " + datasetId + " and organization name of the dataset is " + resource.getOrganizationNameDatasetParent());
@@ -351,29 +379,26 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		}else{
 
-			try{
-				ResourceBean resourceBean = new ResourceBean(
-						resource.getUrl(), 
-						resource.getName(), 
-						resource.getDescription(), 
-						null, 
-						username, 
-						datasetId,
-						null);
 
-				// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
-				String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
-				DataCatalogue catalogue = getCatalogue(scope);
-				String resourceId = catalogue.addResourceToDataset(resourceBean, catalogue.getApiKeyFromUsername(username));
+			ResourceBean resourceBean = new ResourceBean(
+					resource.getUrl(), 
+					resource.getName(), 
+					resource.getDescription(), 
+					null, 
+					username, 
+					datasetId,
+					null);
 
-				if(resourceId != null){
-					logger.debug("Resource  " + resource.getName() + " is now available");
-					// set its id and turn it to the client
-					resource.setOriginalIdInWorkspace(resourceId);
-					return resource;
-				}
-			}catch(Exception e){
-				logger.error("Unable to create new resource", e);
+			// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
+			String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
+			DataCatalogue catalogue = getCatalogue(scope);
+			String resourceId = catalogue.addResourceToDataset(resourceBean, catalogue.getApiKeyFromUsername(username));
+
+			if(resourceId != null){
+				logger.debug("Resource  " + resource.getName() + " is now available");
+				// set its id and turn it to the client
+				resource.setOriginalIdInWorkspace(resourceId);
+				return resource;
 			}
 
 			logger.debug("No resource created");
@@ -454,16 +479,21 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		String toReturn = null;
 
-		if(mapOrganizationScope.containsKey(orgName))
-			toReturn = mapOrganizationScope.get(orgName);
-		else{
-			try{
-				String evaluatedScope = Utils.retrieveScopeFromOrganizationName(orgName);
-				mapOrganizationScope.put(orgName, evaluatedScope);
-				toReturn = evaluatedScope;
-			}catch(Exception e){
-				logger.error("Failed to retrieve scope from OrgName for organization " + orgName, e);
+		if(isWithinPortal()){
+			if(mapOrganizationScope.containsKey(orgName))
+				toReturn = mapOrganizationScope.get(orgName);
+			else{
+				try{
+					String evaluatedScope = Utils.retrieveScopeFromOrganizationName(orgName);
+					mapOrganizationScope.put(orgName, evaluatedScope);
+					toReturn = evaluatedScope;
+				}catch(Exception e){
+					logger.error("Failed to retrieve scope from OrgName for organization " + orgName, e);
+				}
 			}
+		}else{
+			toReturn = "/gcube/preprod/preVRE";
+			mapOrganizationScope.put(orgName, toReturn);
 		}
 		logger.debug("Returning scope " + toReturn);
 		return toReturn;
@@ -505,7 +535,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			}
 		}else{
 			logger.warn("Dev mode detected");
-			toReturn = Arrays.asList(new GroupBean("Group title", "aaa"));
+			toReturn = Arrays.asList();
 		}
 		return toReturn;
 	}
@@ -519,9 +549,9 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		if(!isWithinPortal()){
 			logger.warn("OUT FROM PORTAL DETECTED RETURNING TRUE");
-			return false;
+			return true;
 		}
-		
+
 		try{
 
 			HttpSession httpSession = this.getThreadLocalRequest().getSession();
@@ -570,7 +600,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 			logger.info("Does the user have the right to publish on the catalogue? " + role);
 			return !role.equals(RolesCkanGroupOrOrg.MEMBER);
-			
+
 		}catch(Exception e){
 			logger.error("Failed to check the user's role", e);
 		}
